@@ -3,10 +3,7 @@
 # Parti is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import pygtk
-pygtk.require('2.0')
-
-__version__ = "0.0.7.32"
+__version__ = "0.0.7.36"
 import wimpiggy
 assert wimpiggy.__version__ == __version__
-svn_revision="372"
+svn_revision="493"
