@@ -15,6 +15,7 @@ import cairo
 from wimpiggy.lowlevel import (
                 XGetWindowProperty,         #@UnresolvedImport
                 XChangeProperty,            #@UnresolvedImport
+                NoSuchProperty,             #@UnresolvedImport
                 PropertyError,              #@UnresolvedImport
                 get_xatom, get_pyatom,      #@UnresolvedImport
                 get_xwindow, get_pywindow,  #@UnresolvedImport
@@ -178,7 +179,7 @@ _prop_types = {
                lambda disp, d: d.decode("latin1"),
                "\0"),
     "atom": (str, "ATOM", 32,
-             lambda disp, a: struct.pack("@I", get_xatom(disp, a)),
+             lambda disp, a: struct.pack("@I", get_xatom(a)),
               _get_atom,
              ""),
     "u32": ((int, long), "CARDINAL", 32,
@@ -282,10 +283,14 @@ def prop_get(target, key, type, ignore_errors=False):
         #print(atom)
         data = trap.call_synced(XGetWindowProperty, target, key, atom)
         #print(atom, repr(data[:100]))
+    except NoSuchProperty:
+        log.debug("Missing property %s (%s)", key, type)
+        return None
     except (XError, PropertyError):
         if not ignore_errors:
-            log.info("Missing window or missing property or wrong property type %s (%s)",
-                     key, type, exc_info=True)
+            log.info("Missing window or missing property or wrong property type %s (%s)", key, type)
+            import traceback
+            traceback.print_stack()
         return None
     try:
         return _prop_decode(target, type, data)
