@@ -217,6 +217,13 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
         "geometry": (gobject.TYPE_PYOBJECT,
                      "current (border-corrected, relative to parent) coordinates (x, y, w, h) for the window", "",
                      gobject.PARAM_READABLE),
+        "transient-for": (gobject.TYPE_PYOBJECT,
+                          "Transient for (or None)", "",
+                          gobject.PARAM_READABLE),
+        "window-type": (gobject.TYPE_PYOBJECT,
+                        "Window type",
+                        "NB, most preferred comes first, then fallbacks",
+                        gobject.PARAM_READABLE),
         # NB "notify" signal never fires for the client-contents properties:
         "client-contents": (gobject.TYPE_PYOBJECT,
                             "gtk.gdk.Pixmap containing the window contents", "",
@@ -286,6 +293,26 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
             self._composite.disconnect(self._damage_forward_handle)
             self._composite.destroy()
 
+    def _read_initial_properties(self):
+        transient_for = self.prop_get("WM_TRANSIENT_FOR", "window")
+        # May be None
+        self._internal_set_property("transient-for", transient_for)
+
+        window_types = self.prop_get("_NET_WM_WINDOW_TYPE", ["atom"])
+        if window_types:
+            self._internal_set_property("window-type", window_types)
+        else:
+            if transient_for is not None:
+                # EWMH says that even if it's transient-for, we MUST check to
+                # see if it's override-redirect (and if so treat as NORMAL).
+                # But we wouldn't be here if this was override-redirect.
+                assume_type = "_NET_WM_TYPE_DIALOG"
+            else:
+                assume_type = "_NET_WM_WINDOW_TYPE_NORMAL"
+            self._internal_set_property("window-type",
+                              [gtk.gdk.atom_intern(assume_type)])
+
+
 gobject.type_register(BaseWindowModel)
 
 
@@ -308,6 +335,8 @@ class OverrideRedirectWindowModel(BaseWindowModel):
             # So double check now, *after* putting in our request:
             if not is_mapped(self.client_window):
                 raise Unmanageable, "window already unmapped"
+
+            self._read_initial_properties()
         try:
             trap.call(setup)
         except XError, e:
@@ -316,6 +345,10 @@ class OverrideRedirectWindowModel(BaseWindowModel):
     def do_wimpiggy_unmap_event(self, event):
         self.unmanage()
 
+    def prop_get(self, key, type):
+        return prop_get(self.client_window, key, type, ignore_errors=False)
+
+
 gobject.type_register(OverrideRedirectWindowModel)
 
 
@@ -360,16 +393,9 @@ class WindowModel(BaseWindowModel):
         "class-instance": (gobject.TYPE_PYOBJECT,
                            "Classic X 'class' and 'instance'", "",
                            gobject.PARAM_READABLE),
-        "transient-for": (gobject.TYPE_PYOBJECT,
-                          "Transient for (or None)", "",
-                          gobject.PARAM_READABLE),
         "protocols": (gobject.TYPE_PYOBJECT,
                       "Supported WM protocols", "",
                       gobject.PARAM_READABLE),
-        "window-type": (gobject.TYPE_PYOBJECT,
-                        "Window type",
-                        "NB, most preferred comes first, then fallbacks",
-                        gobject.PARAM_READABLE),
         "pid": (gobject.TYPE_INT,
                 "PID of owning process", "",
                 -1, 65535, -1,
@@ -491,10 +517,12 @@ class WindowModel(BaseWindowModel):
             raise Unmanageable, e
         self._setup_done = True
 
-    def prop_get(self, key, type):
+    def prop_get(self, key, type, ignore_errors=False):
         # Utility wrapper for prop_get on the client_window
         # also allows us to ignore property errors during setup_client
-        return prop_get(self.client_window, key, type, ignore_errors=not self._setup_done)
+        if not self._setup_done:
+            ignore_errors = True
+        return prop_get(self.client_window, key, type, ignore_errors=ignore_errors)
 
     def do_wimpiggy_xkb_event(self, event):
         log("WindowModel.do_wimpiggy_xkb_event(%r)" % event)
@@ -664,7 +692,7 @@ class WindowModel(BaseWindowModel):
             self._property_handlers[name](self)
 
     def _handle_wm_hints(self):
-        wm_hints = self.prop_get("WM_HINTS", "wm-hints")
+        wm_hints = self.prop_get("WM_HINTS", "wm-hints", True)
         if wm_hints is not None:
             # GdkWindow or None
             self._internal_set_property("group-leader", wm_hints.group_leader)
@@ -694,8 +722,8 @@ class WindowModel(BaseWindowModel):
     _property_handlers["WM_NORMAL_HINTS"] = _handle_wm_normal_hints
 
     def _handle_title_change(self):
-        wm_name = self.prop_get("WM_NAME", "latin1")
-        net_wm_name = self.prop_get("_NET_WM_NAME", "utf8")
+        wm_name = self.prop_get("WM_NAME", "latin1", True)
+        net_wm_name = self.prop_get("_NET_WM_NAME", "utf8", True)
         if net_wm_name is not None:
             self._internal_set_property("title", net_wm_name)
         else:
@@ -706,8 +734,8 @@ class WindowModel(BaseWindowModel):
     _property_handlers["_NET_WM_NAME"] = _handle_title_change
 
     def _handle_icon_title_change(self):
-        wm_icon_name = self.prop_get("WM_ICON_NAME", "latin1")
-        net_wm_icon_name = self.prop_get("_NET_WM_ICON_NAME", "utf8")
+        wm_icon_name = self.prop_get("WM_ICON_NAME", "latin1", True)
+        net_wm_icon_name = self.prop_get("_NET_WM_ICON_NAME", "utf8", True)
         if net_wm_icon_name is not None:
             self._internal_set_property("icon-title", net_wm_icon_name)
         else:
@@ -755,22 +783,27 @@ class WindowModel(BaseWindowModel):
 
     def _read_initial_properties(self):
         # Things that don't change:
+        BaseWindowModel._read_initial_properties(self)
+        
         geometry = self.client_window.get_geometry()
         self._internal_set_property("requested-position", (geometry[0], geometry[1]))
         self._internal_set_property("requested-size", (geometry[2], geometry[3]))
 
+        def set_class_instance(s):
+            try:
+                parts = class_instance.split("\0")
+                if len(parts)!=3:
+                    return  False
+                (c, i, _) = parts
+                self._internal_set_property("class-instance", (c, i))
+                return  True
+            except ValueError:
+                log.warn("Malformed WM_CLASS: %s, ignoring", class_instance)
+                return  False
         class_instance = self.prop_get("WM_CLASS", "latin1")
         if class_instance:
-            try:
-                (c, i, _) = class_instance.split("\0")
-            except ValueError:
-                log.warn("Malformed WM_CLASS, ignoring")
-            else:
-                self._internal_set_property("class-instance", (c, i))
-
-        transient_for = self.prop_get("WM_TRANSIENT_FOR", "window")
-        # May be None
-        self._internal_set_property("transient-for", transient_for)
+            if not set_class_instance(class_instance):
+                set_class_instance(self.prop_get("WM_CLASS", "utf8"))
 
         protocols = self.prop_get("WM_PROTOCOLS", ["atom"])
         if protocols is None:
@@ -778,20 +811,6 @@ class WindowModel(BaseWindowModel):
         self._internal_set_property("protocols", protocols)
         self.notify("can-focus")
 
-        window_types = self.prop_get("_NET_WM_WINDOW_TYPE", ["atom"])
-        if window_types:
-            self._internal_set_property("window-type", window_types)
-        else:
-            if self.get_property("transient-for"):
-                # EWMH says that even if it's transient-for, we MUST check to
-                # see if it's override-redirect (and if so treat as NORMAL).
-                # But we wouldn't be here if this was override-redirect.
-                assume_type = "_NET_WM_TYPE_DIALOG"
-            else:
-                assume_type = "_NET_WM_WINDOW_TYPE_NORMAL"
-            self._internal_set_property("window-type",
-                              [gtk.gdk.atom_intern(assume_type)])
-
         pid = self.prop_get("_NET_WM_PID", "u32")
         if pid is not None:
             self._internal_set_property("pid", pid)
