@@ -18,6 +18,6 @@ else:
     raise OSError, "Unknown OS %s" % (_os.name)
 
 def add_notray_option(parser, extra_text=""):
-    parser.add_option("--no-tray", action="store",
-                          dest="no_tray", default=None,
-                          help="Disables the system tray icon%s" % extra_text)
+    parser.add_option("--no-tray", action="store_true",
+                          dest="no_tray", default=False,
+                          help="Disables the system tray%s" % extra_text)
