@@ -1,6 +1,6 @@
 # coding=utf8
 # This file is part of Parti.
-# Copyright (C) 2011 Antoine Martin <antoine@nagafix.co.uk>
+# Copyright (C) 2011, 2012 Antoine Martin <antoine@nagafix.co.uk>
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
 # Parti is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -19,7 +19,7 @@ import datetime
 
 from xpra.platform import XPRA_LOCAL_SERVERS_SUPPORTED
 from xpra.scripts.main import ENCODINGS
-from xpra.keys import XMODMAP_MOD_DEFAULTS, XMODMAP_MOD_ADD, XMODMAP_MOD_CLEAR
+from xpra.keys import XMODMAP_MOD_DEFAULTS, XMODMAP_MOD_ADD, XMODMAP_MOD_CLEAR, get_gtk_keymap
 from wimpiggy.util import gtk_main_quit_really
 from wimpiggy.log import Logger
 log = Logger()
@@ -151,11 +151,14 @@ class ClientExtrasBase(object):
         """ layout, variant, variants"""
         return None,None,None
 
-    def supports_raw_keycodes(self):
-        return False
+    def translate_key(self, depressed, keyval, name, keycode, group, is_modifier, modifiers):
+        return depressed, keyval, name, keycode, group, is_modifier, modifiers
+
+    def get_gtk_keymap(self):
+        return  get_gtk_keymap()
 
     def get_keymap_modifiers(self):
-        return  XMODMAP_MOD_CLEAR, XMODMAP_MOD_ADD
+        return  XMODMAP_MOD_CLEAR, XMODMAP_MOD_ADD, {}, [], []
 
     def get_keymap_spec(self):
         """ xkbmap_print, xkbmap_query, xmodmap """
@@ -180,7 +183,7 @@ class ClientExtrasBase(object):
         dialog.set_name("Xpra")
         from xpra import __version__
         dialog.set_version(__version__)
-        dialog.set_copyright('Copyright (c) 2009-2011')
+        dialog.set_copyright('Copyright (c) 2009-2012')
         dialog.set_authors(('Antoine Martin <antoine@nagafix.co.uk>',
                             'Nathaniel Smith <njs@pobox.com>',
                             'Serviware - Arthur Huillet <ahuillet@serviware.com>'))
@@ -265,13 +268,16 @@ class ClientExtrasBase(object):
             row = add_row(row, gtk.Label("Server Latency"), self.server_latency_label)
             self.client_latency_label = gtk.Label()
             row = add_row(row, gtk.Label("Client Latency"), self.client_latency_label)
-        if self.client.server_start_time>0:
-            self.session_started_label = gtk.Label()
-            row = add_row(row, gtk.Label("Session Started"), self.session_started_label)
+        self.session_started_label = gtk.Label()
+        row = add_row(row, gtk.Label("Session Started"), self.session_started_label)
         self.session_connected_label = gtk.Label()
         row = add_row(row, gtk.Label("Session Connected"), self.session_connected_label)
         self.windows_managed_label = gtk.Label()
         row = add_row(row, gtk.Label("Windows Managed"), self.windows_managed_label)
+        self.regions_sizes_label = gtk.Label()
+        row = add_row(row, gtk.Label("Pixels/region (min/avg/max)"), self.regions_sizes_label)
+        self.regions_per_second_label = gtk.Label()
+        row = add_row(row, gtk.Label("Regions/s"), self.regions_per_second_label)
         self.pixels_per_second_label = gtk.Label()
         row = add_row(row, gtk.Label("Pixels/s"), self.pixels_per_second_label)
 
@@ -306,6 +312,8 @@ class ClientExtrasBase(object):
                     self.client_latency_label.set_text("%sms  (%sms)" % (self.client.client_latency[-1], avg))
             if self.client.server_start_time>0:
                 settimedeltastr(self.session_started_label, self.client.server_start_time)
+            else:
+                self.session_started_label.set_text("unknown")
             settimedeltastr(self.session_connected_label, self.client.start_time)
             real, redirect = 0, 0
             for w in self.client._window_to_id.keys():
@@ -314,32 +322,64 @@ class ClientExtrasBase(object):
                 else:
                     real += 1
             self.windows_managed_label.set_text("%s  (%s transient)" % (real, redirect))
+            regions_sizes = "n/a"
+            regions = "n/a"
             pixels = "n/a"
             if len(self.client.pixel_counter)>0:
-                total = 0
                 now = time.time()
-                mint = now-20       #ignore records older than 20 seconds
-                startt = now        #when we actually start counting from
-                for (t, count) in self.client.pixel_counter:
-                    if t>=mint:
-                        total += count
-                        startt = min(t, startt)
-                if total>0 and startt!=now:
-                    def pixelstr(v):
-                        if v>1000*1000*1000:
-                            return "%sG" % (long(v/1000/1000/100)/10.0)
-                        elif v>1000*1000:
-                            return "%sM" % (long(v/1000/100)/10.0)
-                        elif v>1000:
-                            return "%sK" % (long(v/100)/10.0)
-                        else:
-                            return str(v)
-                    t, last = self.client.pixel_counter[-1]
-                    if t<now-5:
-                        last = 0
-                    avg = long(total/(now-startt))
-                    pixels = "%s  (%s)" % (pixelstr(last), pixelstr(avg))
+                def pixelstr(v):
+                    if v<0:
+                        return  "n/a"
+                    if v>1000*1000*1000:
+                        return "%sG" % (long(v/1000/1000/100)/10.0)
+                    elif v>1000*1000:
+                        return "%sM" % (long(v/1000/100)/10.0)
+                    elif v>1000:
+                        return "%sK" % (long(v/100)/10.0)
+                    else:
+                        return str(v)
+                def fpsstr(v):
+                    if v<0:
+                        return  "n/a"
+                    return "%s" % (long(v*100)/100.0)
+                def average(seconds):
+                    total = 0
+                    total_n = 0
+                    mins = None
+                    maxs = 0
+                    avgs = 0
+                    mint = now-seconds      #ignore records older than N seconds
+                    startt = now            #when we actually start counting from
+                    for (t, count) in self.client.pixel_counter:
+                        if t>=mint:
+                            total += count
+                            total_n += 1
+                            startt = min(t, startt)
+                            if mins:
+                                mins = min(mins,count)
+                            else:
+                                mins = count
+                            maxs = max(maxs, count)
+                            avgs += count
+                    if total==0 or startt==now:
+                        return  None
+                    avgs = avgs/total_n
+                    elapsed = now-startt
+                    return long(total/elapsed), total_n/elapsed, mins, avgs, maxs
+                p20 = average(20)
+                if p20:
+                    avg20,fps20,mins,avgs,maxs = p20
+                    p1 = average(1)
+                    if p1:
+                        avg1,fps1 = average(1)[:2]
+                    else:
+                        avg1,fps1 = -1, -1
+                    pixels = "%s  (%s)" % (pixelstr(avg1), pixelstr(avg20))
+                    regions = "%s  (%s)" % (fpsstr(fps1), fpsstr(fps20))
+                    regions_sizes = "%s  %s  %s" % (pixelstr(mins), pixelstr(avgs), pixelstr(maxs))
 
+            self.regions_sizes_label.set_text(regions_sizes)
+            self.regions_per_second_label.set_text(regions)
             self.pixels_per_second_label.set_text(pixels)
             return True
         gobject.timeout_add(1000, populate_table)
@@ -375,7 +415,7 @@ class ClientExtrasBase(object):
 
 
 
-    def grok_modifier_map(self, display_source):
+    def grok_modifier_map(self, display_source, xkbmap_mod_meanings):
         modifier_map = {
             "shift": 1 << 0,
             "lock": 1 << 1,
@@ -385,18 +425,14 @@ class ClientExtrasBase(object):
             "mod3": 1 << 5,
             "mod4": 1 << 6,
             "mod5": 1 << 7,
-            "scroll": 0,
-            "num": 0,
-            "meta": 0,
-            "super": 0,
-            "hyper": 0,
-            "alt": 0,
             }
-        modifier_map["nuisance"] = (modifier_map["lock"]
-                                    | modifier_map["scroll"]
-                                    | modifier_map["num"])
         return modifier_map
 
+    def current_modifiers(self, modifiers):
+        """ subclasses may filter or add modifiers (see win32)
+            this default implementation just leaves them unchanged """
+        return  modifiers
+
 
     def get_data_dir(self):
         return  os.path.dirname(sys.executable) or os.getcwd()
@@ -536,6 +572,11 @@ class ClientExtrasBase(object):
         encodings.set_submenu(self.encodings_submenu)
         self.popup_menu_workaround(self.encodings_submenu)
         def set_encodingsmenuitem(*args):
+            if self.client.mmap_enabled:
+                #mmap disables encoding and uses raw rgb24
+                encodings.set_label("Encoding")
+                encodings.set_tooltip_text("memory mapped transfers are in use so picture encoding is disabled")
+                encodings.set_sensitive(False)
             for encoding in ENCODINGS:
                 encoding_item = gtk.CheckMenuItem(encoding)
                 encoding_item.get_label()
@@ -608,6 +649,11 @@ class ClientExtrasBase(object):
                     #no variants:
                     self.layout_submenu.append(kbitem(name, layout, None))
         def set_selected_layout(*args):
+            if self.client._raw_keycodes_full:
+                #server supports raw keycode and we do too
+                #so no need to let the user select the keymap
+                keyboard.hide()
+                return
             layout = self.client.xkbmap_layout
             variant = self.client.xkbmap_variant
             def is_match(checkitem):
@@ -651,6 +697,29 @@ class ClientExtrasBase(object):
             self.jpeg_quality.set_sensitive("jpeg"==self.client.encoding)
             self.updated_menus()
 
+    def make_compressionmenu(self):
+        self.compression = self.menuitem("Compression", "compressed.png", "Network packet compression", None)
+        self.compression_submenu = gtk.Menu()
+        self.compression.set_submenu(self.compression_submenu)
+        self.popup_menu_workaround(self.compression_submenu)
+        compression_options = {0 : "None"}
+        def set_compression(item):
+            item = ensure_item_selected(self.compression_submenu, item)
+            c = int(item.get_label().replace("None", "0"))
+            if c!=self.client.compression_level:
+                log.debug("setting compression level to %s", c)
+                self.client.compression_level = c
+                self.client.send_deflate_level()
+        for i in range(0, 10):
+            c = gtk.CheckMenuItem(str(compression_options.get(i, i)))
+            c.set_draw_as_radio(True)
+            c.set_active(i==self.client.compression_level)
+            c.connect('activate', set_compression)
+            self.compression_submenu.append(c)
+        self.compression_submenu.show_all()
+        return self.compression
+
+
     def updated_menus(self):
         """ subclasses may override this method - see darwin """
         pass
@@ -688,7 +757,8 @@ class ClientExtrasBase(object):
         menu.append(gtk.SeparatorMenuItem())
         menu.append(self.make_bellmenuitem())
         menu.append(self.make_notificationsmenuitem())
-        menu.append(self.make_clipboardmenuitem())
+        if not self.client.readonly:
+            menu.append(self.make_clipboardmenuitem())
         if len(ENCODINGS)>1:
             menu.append(self.make_encodingsmenuitem())
         else:
@@ -698,7 +768,9 @@ class ClientExtrasBase(object):
         else:
             self.jpeg_quality = None
             self.jpeg_submenu = None
-        menu.append(self.make_layoutsmenuitem())
+        menu.append(self.make_compressionmenu())
+        if not self.client.readonly:
+            menu.append(self.make_layoutsmenuitem())
         menu.append(self.make_refreshmenuitem())
         menu.append(self.make_raisewindowsmenuitem())
         #menu.append(item("Options", "configure", None, self.options))
