@@ -1,5 +1,5 @@
 # This file is part of Parti.
-# Copyright (C) 2011 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011, 2012 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2009, 2010 Nathaniel Smith <njs@pobox.com>
 # Parti is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -13,11 +13,12 @@ gobject.threads_init()
 import os
 import socket # for socket.error
 import zlib
+import errno
 
 from Queue import Queue
 from threading import Thread, Lock
 
-from xpra.bencode import bencode, IncrBDecode
+from xpra.bencode import bencode, bdecode
 
 from wimpiggy.log import Logger
 log = Logger()
@@ -26,14 +27,22 @@ log = Logger()
 # (os.read/os.write-style interface) two-way byte stream:
 
 class TwoFileConnection(object):
-    def __init__(self, writeable, readable):
+    def __init__(self, writeable, readable, abort_test=None):
         self._writeable = writeable
         self._readable = readable
+        self._abort_test = abort_test
+
+    def may_abort(self, action):
+        """ if abort_test is defined, run it """
+        if self._abort_test:
+            self._abort_test(action)
 
     def read(self, n):
+        self.may_abort("read")
         return os.read(self._readable.fileno(), n)
 
     def write(self, buf):
+        self.may_abort("write")
         return os.write(self._writeable.fileno(), buf)
 
     def close(self):
@@ -62,6 +71,14 @@ def repr_ellipsized(obj, limit=100):
 def dump_packet(packet):
     return "[" + ", ".join([repr_ellipsized(str(x), 50) for x in packet]) + "]"
 
+def untilConcludes(f, *a, **kw):
+    while True:
+        try:
+            return f(*a, **kw)
+        except (IOError, OSError), e:
+            if e.args[0] == errno.EINTR:
+                continue
+            raise
 
 class Protocol(object):
     CONNECTION_LOST = object()
@@ -74,12 +91,11 @@ class Protocol(object):
         self._read_queue = Queue(5)
         # Invariant: if .source is None, then _source_has_more == False
         self.source = None
-        self.jpegquality = 0
         self._source_has_more = False
         self._recv_counter = 0
         self._send_size = False
         self._closed = False
-        self._read_decoder = IncrBDecode()
+        self._read_buffer = ""
         self._compressor = None
         self._decompressor = None
         self._write_thread = Thread(target=self._write_thread_loop)
@@ -122,15 +138,49 @@ class Protocol(object):
         if c:
             self._write_queue.put(c)
 
+    def verify_packet(self, packet):
+        """ look for None values which may have caused the packet to fail encoding """
+        if type(packet)!=list:
+            return
+        assert len(packet)>0
+        tree = ["'%s' packet" % packet[0]]
+        self.do_verify_packet(tree, packet)
+
+    def do_verify_packet(self, tree, packet):
+        def err(msg):
+            log.error("%s in %s", msg, "->".join(tree))
+        def new_tree(append):
+            nt = tree[:]
+            nt.append(append)
+            return nt
+        if packet is None:
+            return err("None value")
+        if type(packet)==list:
+            i = 0
+            for x in packet:
+                self.do_verify_packet(new_tree("[%s]" % i), x)
+                i += 1
+        elif type(packet)==dict:
+            for k,v in packet.items():
+                self.do_verify_packet(new_tree("key for value='%s'" % str(v)), k)
+                self.do_verify_packet(new_tree("value for key='%s'" % str(k)), v)
+
     def _flush_one_packet_into_buffer(self):
         if not self.source:
             return
         packet, self._source_has_more = self.source.next_packet()
         if packet is not None:
-            #log("writing %s", dump_packet(packet), type="raw.write")
+            self._add_packet_to_queue(packet)
+
+    def _add_packet_to_queue(self, packet):
+        try:
             data = bencode(packet)
-            l = len(data)
-            self._write_lock.acquire()
+        except KeyError or TypeError, e:
+            self.verify_packet(packet)
+            raise e
+        l = len(data)
+        self._write_lock.acquire()
+        try:
             try:
                 if self._send_size:
                     if l<=1024:
@@ -140,7 +190,15 @@ class Protocol(object):
                     self._queue_write("PS%014d" % l)
                 self._queue_write(data, True)
             finally:
-                self._write_lock.release()
+                if packet[0]=="set_deflate":
+                    level = packet[1]
+                    log("set_deflate packet, changing compressor to level=%s", level)
+                    if level==0:
+                        self._compressor = None
+                    else:
+                        self._compressor = zlib.compressobj(level)
+        finally:
+            self._write_lock.release()
 
     def _write_thread_loop(self):
         try:
@@ -151,8 +209,9 @@ class Protocol(object):
                     log("write thread: empty marker, exiting")
                     break
                 try:
-                    while buf:
-                        buf = buf[self._conn.write(buf):]
+                    while buf and not self._closed:
+                        written = untilConcludes(self._conn.write, buf)
+                        buf = buf[written:]
                 except (OSError, IOError, socket.error), e:
                     self._call_connection_lost("Error writing to connection: %s" % e)
                     break
@@ -169,7 +228,7 @@ class Protocol(object):
         try:
             while not self._closed:
                 try:
-                    buf = self._conn.read(8192)
+                    buf = untilConcludes(self._conn.read, 8192)
                 except (ValueError, OSError, IOError, socket.error), e:
                     self._call_connection_lost("Error reading from connection: %s" % e)
                     return
@@ -183,7 +242,8 @@ class Protocol(object):
                     log("read thread: eof")
                     break
         finally:
-            log("read thread: ended")
+            log("read thread: ended, closing socket")
+            self._conn.close()
 
     def _call_connection_lost(self, message="", exc_info=False):
         gobject.idle_add(self._connection_lost, message, exc_info)
@@ -197,34 +257,75 @@ class Protocol(object):
 
     def _read_parse_thread_loop(self):
         try:
+            current_packet_size = -1
             while not self._closed:
                 buf = self._read_queue.get()
                 if not buf:
                     return self._call_connection_lost("empty marker in read queue")
                 if self._decompressor is not None:
                     buf = self._decompressor.decompress(buf)
-                try:
-                    self._read_decoder.add(buf)
-                except:
-                    return self._call_connection_lost("read buffer is in an inconsistent state, cannot continue", exc_info=True)
-                while not self._closed:
+                if self._read_buffer:
+                    self._read_buffer = self._read_buffer + buf
+                else:
+                    self._read_buffer = buf
+                while not self._closed and len(self._read_buffer)>0:
                     had_deflate = (self._decompressor is not None)
                     try:
-                        result = self._read_decoder.process()
+                        if current_packet_size<0 and len(self._read_buffer)>0 and self._read_buffer[0]=="P":
+                            #spotted packet size header
+                            if len(self._read_buffer)<16:
+                                break   #incomplete
+                            current_packet_size = int(self._read_buffer[2:16])
+                            self._read_buffer = self._read_buffer[16:]
+
+                        if current_packet_size>0 and len(self._read_buffer)<current_packet_size:
+                            log.debug("incomplete packet: only %s of %s bytes received", len(self._read_buffer), current_packet_size)
+                            break
+
+                        result = bdecode(self._read_buffer)
                     except ValueError:
-                        # Peek at the data we got, in case we can make sense of it:
-                        self._process_packet([Protocol.GIBBERISH, self._read_decoder.unprocessed()])
-                        # Then hang up:
-                        return self._call_connection_lost("gibberish received")
-                    if result is None:
+                        #could be a partial packet (without size header)
+                        #or could be just a broken packet...
+                        def packet_error(buf):
+                            if self._closed:
+                                return
+                            # Peek at the data we got, in case we can make sense of it:
+                            self._process_packet([Protocol.GIBBERISH, buf])
+                            # Then hang up:
+                            return self._connection_lost("gibberish received: %s" % repr_ellipsized(buf))
+
+                        if current_packet_size>0:
+                            #we had the size, so the packet should have been valid!
+                            packet_error(self._read_buffer)
+                            return
+                        else:
+                            #wait a little before deciding
+                            #unsized packets are either old clients (don't really care about them)
+                            #or hello packets (small-ish)
+                            def check_error_state(old_buffer):
+                                if old_buffer==self._read_buffer:
+                                    packet_error(self._read_buffer)
+                            gobject.timeout_add_seconds(1000, check_error_state, self._read_buffer)
+                            break
+
+                    current_packet_size = -1
+                    if result is None or self._closed:
                         break
-                    packet, unprocessed = result
+                    packet, l = result
                     gobject.idle_add(self._process_packet, packet)
+                    unprocessed = self._read_buffer[l:]
+                    if packet[0]=="set_deflate":
+                        level = packet[1]
+                        log("set_deflate packet, changing decompressor to level=%s", level)
+                        if level==0:
+                            self._decompressor = None
+                        else:
+                            self._decompressor = zlib.decompressobj()
                     if not had_deflate and (self._decompressor is not None):
                         # deflate was just enabled: so decompress the unprocessed
                         # data
                         unprocessed = self._decompressor.decompress(unprocessed)
-                    self._read_decoder = IncrBDecode(unprocessed)
+                    self._read_buffer = unprocessed
         finally:
             log("read parse thread: ended")
 
@@ -234,7 +335,6 @@ class Protocol(object):
                      " allegedly closed (%s)", dump_packet(decoded))
             return
         try:
-            log("got %s", dump_packet(decoded), type="raw.read")
             self._process_packet_cb(self, decoded)
         except KeyboardInterrupt:
             raise
@@ -244,6 +344,7 @@ class Protocol(object):
             # Ignore and continue, maybe things will work out anyway
         return False
 
+
     def enable_deflate(self, level):
         assert self._compressor is None and self._decompressor is None
         # Flush everything out of the source
@@ -253,9 +354,29 @@ class Protocol(object):
         self._compressor = zlib.compressobj(level)
         self._decompressor = zlib.decompressobj()
 
+    def flush_then_close(self, last_packet):
+        self._add_packet_to_queue(last_packet)
+        self.terminate_io_threads()
+        #wait for last_packet to be sent:
+        def wait_for_end_of_write(timeout=15):
+            log.debug("wait_for_end_of_write(%s) closed=%s, size=%s", timeout, self._closed, self._write_queue.qsize())
+            if self._closed:
+                """ client has disconnected """
+                return
+            if self._write_queue.empty() or timeout<=0:
+                """ threads have terminated or we timedout """
+                self.close()
+            else:
+                """ check again soon: """
+                gobject.timeout_add(200, wait_for_end_of_write, timeout-1)
+        wait_for_end_of_write()
+
     def close(self):
         if not self._closed:
             self._closed = True
+        self.terminate_io_threads()
+
+    def terminate_io_threads(self):
         #make the threads exit by adding the empty marker:
         self._write_queue.put(None)
         try:
