@@ -1,6 +1,6 @@
 # This file is part of Parti.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2011 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2012 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Parti is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -60,12 +60,15 @@ def main(script_file, cmdline):
         start_str = ""
         list_str = ""
         upgrade_str = ""
-        note_str = "(This xpra install does not support starting local servers.)"
+        note_str = "(This xpra installation does not support starting local servers.)"
         stop_str = ""
     parser = OptionParser(version="xpra v%s" % xpra.__version__,
                           usage="".join(["\n",
                                          start_str,
                                          "\t%prog attach [DISPLAY]\n",
+                                         "\t%prog detach [DISPLAY]\n",
+                                         "\t%prog screenshot filename [DISPLAY]\n",
+                                         "\t%prog version [DISPLAY]\n",
                                          stop_str,
                                          list_str,
                                          upgrade_str,
@@ -102,6 +105,15 @@ def main(script_file, cmdline):
     parser.add_option("--no-keyboard-sync", action="store_false",
                       dest="keyboard_sync", default=True,
                       help="Disable keyboard state synchronization, prevents keys from repeating on high latency links but also may disrupt applications which access the keyboard directly")
+    # let the platform specific code add its own options:
+    # adds "--no-tray" for platforms that support it
+    add_client_options(parser)
+    parser.add_option("--no-mmap", action="store_false",
+                      dest="mmap", default=True,
+                      help="Disable memory mapped transfers for local connections")
+    parser.add_option("--disable-mmap", action="store_false",
+                      dest="mmap", default=True,
+                      help="Disable memory mapped transfers for local connections")
     parser.add_option("--password-file", action="store",
                       dest="password_file", default=None,
                       help="The file containing the password required to connect (useful to secure TCP mode)")
@@ -142,9 +154,9 @@ def main(script_file, cmdline):
                       dest="key_shortcuts", type="str", default=[],
                       help="Define key shortcuts that will trigger specific actions."
                       + " Defaults to Meta+Shift+F4:quit if no shortcuts are defined.")
-    parser.add_option("--disable-mmap", action="store_false",
-                      dest="mmap", default=True,
-                      help="Disable memory mapped transfers for local connections")
+    parser.add_option("--readonly", action="store_true",
+                      dest="readonly", default=False,
+                      help="Ignore all keyboard input and mouse events from client")
     parser.add_option("-z", "--compress", action="store",
                       dest="compression_level", type="int", default=3,
                       metavar="LEVEL",
@@ -161,8 +173,6 @@ def main(script_file, cmdline):
     parser.add_option("-d", "--debug", action="store",
                       dest="debug", default=None, metavar="FILTER1,FILTER2,...",
                       help="List of categories to enable debugging for (or \"all\")")
-    # let the platform specific code add its own options:
-    add_client_options(parser)
     (options, args) = parser.parse_args(cmdline[1:])
     if "jpeg" not in ENCODINGS:
         #ensure the default values are set even though
@@ -188,28 +198,38 @@ def main(script_file, cmdline):
             else:
                 logger = logging.getLogger(cat)
             logger.setLevel(level)
-    def toggle_debug_on(*args):
-        toggle_logging(logging.DEBUG)
-    def toggle_debug_off(*args):
-        toggle_logging(logging.INFO)
-        
+
+    def dump_frames(*arsg):
+        import traceback
+        frames = sys._current_frames()
+        print("")
+        print("found %s frames:" % len(frames))
+        for fid,frame in frames.items():
+            print("%s - %s:" % (fid, frame))
+            traceback.print_stack(frame)
+        print("")
+
     if options.debug is not None:
-        toggle_debug_on()
+        toggle_logging(logging.DEBUG)
     else:
-        toggle_debug_off()
+        toggle_logging(logging.INFO)
     logging.root.addHandler(logging.StreamHandler(sys.stderr))
     if os.name=="posix":
-        signal.signal(signal.SIGUSR1, toggle_debug_on)
-        signal.signal(signal.SIGUSR2, toggle_debug_off)
+        def sigusr1(*args):
+            dump_frames()
+            toggle_logging(logging.DEBUG)
+        def sigusr2(*args):
+            toggle_logging(logging.INFO)
+        signal.signal(signal.SIGUSR1, sigusr1)
+        signal.signal(signal.SIGUSR2, sigusr2)
 
     mode = args.pop(0)
-
     if mode in ("start", "upgrade") and XPRA_LOCAL_SERVERS_SUPPORTED:
         nox()
         from xpra.scripts.server import run_server
         run_server(parser, options, mode, script_file, args)
-    elif mode == "attach" or mode == "detach":
-        run_client(parser, options, args, mode=="detach")
+    elif mode in ("attach", "detach", "screenshot", "version"):
+        run_client(parser, options, args, mode)
     elif mode == "stop" and XPRA_LOCAL_SERVERS_SUPPORTED:
         nox()
         run_stop(parser, options, args)
@@ -260,7 +280,7 @@ def parse_display_name(parser, opts, display_name):
             desc["host"] = "127.0.0.1"
         return desc
     else:
-        parser.error("unknown format for display name")
+        parser.error("unknown format for display name: %s" % display_name)
 
 def pick_display(parser, opts, extra_args):
     if len(extra_args) == 0:
@@ -298,7 +318,16 @@ def connect_or_fail(display_desc):
             child = Popen(cmd, stdin=PIPE, stdout=PIPE)
         except OSError, e:
             sys.exit("Error running ssh program '%s': %s" % (cmd[0], e))
-        return TwoFileConnection(child.stdin, child.stdout)
+        def abort_test(action):
+            """ if ssh dies, we don't need to try to read/write from its sockets """
+            e = child.poll()
+            if e is not None:
+                error_message = "cannot %s using %s: the SSH process has terminated with exit code=%s" % (action, display_desc["full_ssh"], e)
+                print(error_message)
+                from wimpiggy.util import gtk_main_quit_really
+                gtk_main_quit_really()
+                raise IOError(error_message)
+        return TwoFileConnection(child.stdin, child.stdout, abort_test)
 
     elif XPRA_LOCAL_SERVERS_SUPPORTED and display_desc["type"] == "unix-domain":
         sockdir = DotXpra()
@@ -315,8 +344,11 @@ def connect_or_fail(display_desc):
         assert False, "unsupported display type in connect"
 
 
-def run_client(parser, opts, extra_args, detach):
-    from xpra.client import XpraClient
+def run_client(parser, opts, extra_args, mode):
+    if mode=="screenshot":
+        screenshot_filename = extra_args[0]
+        extra_args = extra_args[1:]
+
     conn = connect_or_fail(pick_display(parser, opts, extra_args))
     if opts.compression_level < 0 or opts.compression_level > 9:
         parser.error("Compression level must be between 0 and 9 inclusive.")
@@ -325,10 +357,15 @@ def run_client(parser, opts, extra_args, detach):
     if opts.title_suffix is not None and opts.title!="@title@ on @client-machine@":
         parser.error("use --title or --title-suffix but not both!")
 
-    app = XpraClient(conn, opts)
-    def handshake_complete_msg(*args):
-        sys.stdout.write("Attached (press Control-C to detach)\n")
-        sys.stdout.flush()
+    if mode=="screenshot":
+        from xpra.client_base import ScreenshotXpraClient
+        app = ScreenshotXpraClient(conn, opts, screenshot_filename)
+    elif mode=="version":
+        from xpra.client_base import VersionXpraClient
+        app = VersionXpraClient(conn, opts)
+    else:
+        from xpra.client import XpraClient
+        app = XpraClient(conn, opts)
     def got_gibberish_msg(obj, data):
         if "assword" in data:
             sys.stdout.write("Your ssh program appears to be asking for a password.\n"
@@ -338,13 +375,15 @@ def run_client(parser, opts, extra_args, detach):
             sys.stdout.write("Your ssh program appears to be asking for a username.\n"
                              "Perhaps try using something like 'ssh:USER@host:display'?\n")
             sys.stdout.flush()
-    app.connect("handshake-complete", handshake_complete_msg)
     app.connect("received-gibberish", got_gibberish_msg)
-    if detach:
-        def do_detach(*args):
+    def handshake_complete(*args):
+        if mode=="detach":
             sys.stdout.write("handshake-complete: detaching")
             app.quit()
-        app.connect("handshake-complete", do_detach)
+        elif mode=="attach":
+            sys.stdout.write("Attached (press Control-C to detach)\n")
+            sys.stdout.flush()
+    app.connect("handshake-complete", handshake_complete)
     signal.signal(signal.SIGINT, app.quit)
     try:
         app.run()
