@@ -1,5 +1,5 @@
 # This file is part of Parti.
-# Copyright (C) 2010, 2011 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010, 2012 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Parti is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -24,8 +24,8 @@ def run_cleanups():
     for c in _cleanups:
         try:
             c()
-        except:
-            pass
+        except Exception, e:
+            print("error running cleanup %s: %s" % (c, e))
 
 def deadly_signal(signum, frame):
     print("got signal %s, exiting" % signum)
@@ -98,7 +98,13 @@ def sh_quotemeta(s):
 def xpra_runner_shell_script(xpra_file, starting_dir):
     script = []
     script.append("#!/bin/sh\n")
-    for var, value in os.environ.iteritems():
+    for var, value in os.environ.items():
+        # these aren't used by xpra, and some should not be exposed
+        # as they are either irrelevant or simply do not match
+        # the new environment used by xpra
+        # TODO: use a whitelist
+        if var in ["XDG_SESSION_COOKIE", "LS_COLORS"]:
+            continue
         # :-separated envvars that people might change while their server is
         # going:
         if var in ("PATH", "LD_LIBRARY_PATH", "PYTHONPATH"):
@@ -191,10 +197,11 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     # daemonize:
     starting_dir = os.getcwd()
 
+    clobber = upgrading or opts.use_display
     # Daemonize:
     if opts.daemon:
         try:
-            logpath = dotxpra.server_socket_path(display_name, upgrading) + ".log"
+            logpath = dotxpra.server_socket_path(display_name, clobber) + ".log"
         except ServerSockInUse:
             parser.error("You already have an xpra server running at %s\n"
                          "  (did you want 'xpra upgrade'?)"
@@ -228,17 +235,18 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
 
     # Write out a shell-script so that we can start our proxy in a clean
     # environment:
-    open(scriptpath, "w").write(xpra_runner_shell_script(xpra_file,
-                                                         starting_dir))
+    scriptfile = open(scriptpath, "w")
     # Unix is a little silly sometimes:
     umask = os.umask(0)
     os.umask(umask)
-    os.chmod(scriptpath, 0777 & ~umask)
+    os.fchmod(scriptfile.fileno(), 0700 & ~umask)
+    scriptfile.write(xpra_runner_shell_script(xpra_file, starting_dir))
+    scriptfile.close()
 
     # Do this after writing out the shell script:
     os.environ["DISPLAY"] = display_name
 
-    if not upgrading and not opts.use_display:
+    if not clobber:
         # We need to set up a new server environment
         xauthority = os.environ.get("XAUTHORITY",
                                     os.path.expanduser("~/.Xauthority"))
@@ -274,40 +282,49 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         default_display.close()
     manager.set_default_display(display)
 
-    if not opts.use_display:
-        if upgrading:
-            xvfb_pid = get_pid()
-        else:
-            xvfb_pid = xvfb.pid
-
-        def kill_xvfb():
-            # Close our display(s) first, so the server dying won't kill us.
-            for display in gtk.gdk.display_manager_get().list_displays():
-                display.close()
-            if xvfb_pid is not None:
-                os.kill(xvfb_pid, signal.SIGTERM)
-        _cleanups.append(kill_xvfb)
+    if clobber:
+        xvfb_pid = get_pid()
+    else:
+        xvfb_pid = xvfb.pid
 
+    def kill_xvfb():
+        # Close our display(s) first, so the server dying won't kill us.
+        for display in gtk.gdk.display_manager_get().list_displays():
+            display.close()
         if xvfb_pid is not None:
-            save_pid(xvfb_pid)
+            os.kill(xvfb_pid, signal.SIGTERM)
+    _cleanups.append(kill_xvfb)
+
+    if xvfb_pid is not None:
+        save_pid(xvfb_pid)
 
     sockets = []
-    sockpath = dotxpra.server_socket_path(display_name, upgrading)
+    sockpath = dotxpra.server_socket_path(display_name, clobber)
+    #print("creating server socket %s" % sockpath)
     sockets.append(create_unix_domain_socket(sockpath))
-    if opts.bind_tcp:
-        sockets.append(create_tcp_socket(parser, opts.bind_tcp))
-
-    # This import is delayed because the module depends on gtk:
-    import xpra.server
-    app = xpra.server.XpraServer(upgrading, sockets, opts.session_name,
-                                 opts.password_file, opts.pulseaudio, opts.clipboard, opts.randr, opts.encoding, opts.mmap)
-    def cleanup_socket(self):
-        print("removing socket")
+    def cleanup_socket():
+        print("removing socket %s" % sockpath)
         try:
             os.unlink(sockpath)
         except:
             pass
     _cleanups.append(cleanup_socket)
+    tcp_socket = None
+    if opts.bind_tcp:
+        tcp_socket = create_tcp_socket(parser, opts.bind_tcp)
+        sockets.append(tcp_socket)
+    def cleanup_tcp_socket():
+        if tcp_socket:
+            print("closing tcp socket")
+            try:
+                tcp_socket.close()
+            except:
+                pass
+    _cleanups.append(cleanup_tcp_socket)
+
+    # This import is delayed because the module depends on gtk:
+    import xpra.server
+    app = xpra.server.XpraServer(clobber, sockets, opts)
 
     child_reaper = ChildReaper(app, opts.exit_with_children)
     # Always register the child reaper, because even if exit_with_children is
@@ -334,5 +351,8 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
 
     _cleanups.append(app.cleanup)
     if app.run():
+        print("upgrading: not cleaning up Xvfb or socket")
         # Upgrading, so leave X server running
+        # and don't delete the new socket (not ours)
         _cleanups.remove(kill_xvfb)
+        _cleanups.remove(cleanup_socket)
