@@ -11,7 +11,7 @@ os.environ["PLINK_PROTOCOL"] = "ssh"
 DEFAULT_SSH_CMD = "plink"
 GOT_PASSWORD_PROMPT_SUGGESTION = \
    'Perhaps you need to set up Pageant, or (less secure) use --ssh="plink -pw YOUR-PASSWORD"?\n'
-X11_KEYMAPS = False
 
 def add_client_options(parser):
-    pass
+    from xpra.platform import add_notray_option
+    add_notray_option(parser, ", this will also disable notifications!")
