@@ -15,10 +15,6 @@ import gtk
 _display = gtk.gdk.get_display()
 assert _display, "cannot open the display with GTK, is DISPLAY set?"
 
-from wimpiggy.keys import grok_modifier_map
-from xpra.xposix.xclipboard import ClipboardProtocolHelper
-from xpra.xposix.xsettings import XSettingsWatcher
-from xpra.xposix.xroot_props import XRootPropWatcher
 from xpra.platform.client_extras_base import ClientExtrasBase
 
 from wimpiggy.log import Logger
@@ -34,8 +30,13 @@ class ClientExtras(ClientExtrasBase):
         self.setup_x11_bell()
         self.has_dbusnotify = False
         self.has_pynotify = False
-        self.setup_dbusnotify() or self.setup_pynotify()
-        self.setup_clipboard_helper(ClipboardProtocolHelper)
+        if not self.setup_dbusnotify() and not self.setup_pynotify():
+            log.error("turning notifications off")
+        try:
+            from xpra.xposix.xclipboard import ClipboardProtocolHelper
+            self.setup_clipboard_helper(ClipboardProtocolHelper)
+        except ImportError:
+            self.clipboard_helper = None
 
     def exit(self):
         ClientExtrasBase.exit(self)
@@ -163,14 +164,14 @@ class ClientExtras(ClientExtrasBase):
         try:
             import dbus.glib
             assert dbus.glib
-            bus = dbus.SessionBus()
+            self.dbus_session = dbus.SessionBus()
             FD_NOTIFICATIONS = 'org.freedesktop.Notifications'
-            obj = bus.get_object(FD_NOTIFICATIONS, '/org/freedesktop/Notifications')
-            self.dbusnotify = dbus.Interface(obj, FD_NOTIFICATIONS)
+            self.org_fd_notifications = self.dbus_session.get_object(FD_NOTIFICATIONS, '/org/freedesktop/Notifications')
+            self.dbusnotify = dbus.Interface(self.org_fd_notifications, FD_NOTIFICATIONS)
             self.has_dbusnotify = True
-            log.info("using dbusnotify: %s(%s)", type(self.dbusnotify), FD_NOTIFICATIONS)
+            log("using dbusnotify: %s(%s)", type(self.dbusnotify), FD_NOTIFICATIONS)
         except Exception, e:
-            log.error("cannot import pynotify wrapper (turning notifications off) : %s", e)
+            log.error("cannot import dbus.glib notification wrapper: %s", e)
         return self.has_dbusnotify
 
     def setup_pynotify(self):
@@ -179,9 +180,9 @@ class ClientExtras(ClientExtrasBase):
             import pynotify
             pynotify.init("Xpra")
             self.has_pynotify = True
-            log.info("using pynotify: %s", pynotify)
+            log("using pynotify: %s", pynotify)
         except ImportError, e:
-            log.error("cannot import pynotify wrapper (turning notifications off) : %s", e)
+            log.error("cannot import pynotify wrapper: %s", e)
         return self.has_pynotify
 
     def setup_x11_bell(self):
@@ -203,12 +204,17 @@ class ClientExtras(ClientExtrasBase):
             self.ROOT_PROPS["PULSE_SERVER"] = "pulse-server"
         def setup_xprop_xsettings(client):
             log.debug("setup_xprop_xsettings(%s)", client)
-            self._xsettings_watcher = XSettingsWatcher()
-            self._xsettings_watcher.connect("xsettings-changed", self._handle_xsettings_changed)
-            self._handle_xsettings_changed()
-            self._root_props_watcher = XRootPropWatcher(self.ROOT_PROPS.keys())
-            self._root_props_watcher.connect("root-prop-changed", self._handle_root_prop_changed)
-            self._root_props_watcher.notify_all()
+            try:
+                from xpra.xposix.xsettings import XSettingsWatcher
+                from xpra.xposix.xroot_props import XRootPropWatcher
+                self._xsettings_watcher = XSettingsWatcher()
+                self._xsettings_watcher.connect("xsettings-changed", self._handle_xsettings_changed)
+                self._handle_xsettings_changed()
+                self._root_props_watcher = XRootPropWatcher(self.ROOT_PROPS.keys())
+                self._root_props_watcher.connect("root-prop-changed", self._handle_root_prop_changed)
+                self._root_props_watcher.notify_all()
+            except ImportError, e:
+                log.error("failed to load X11 properties/settings bindings: %s - root window properties will not be propagated", e)
         self.client.connect("handshake-complete", setup_xprop_xsettings)
 
     def _handle_xsettings_changed(self, *args):
@@ -279,12 +285,33 @@ class ClientExtras(ClientExtrasBase):
         return None
 
     def get_keymap_modifiers(self):
+        try:
+            from wimpiggy.lowlevel import get_modifier_mappings         #@UnresolvedImport
+            mod_mappings = get_modifier_mappings()
+            if mod_mappings:
+                #ie: {"shift" : ["Shift_L", "Shift_R"], "mod1" : "Meta_L", ...]}
+                log.debug("modifier mappings=%s", mod_mappings)
+                meanings = {}
+                clear = []
+                add = []
+                for modifier,keynames in mod_mappings.items():
+                    clear.append("clear %s" % modifier)
+                    add.append("add %s = %s" % (modifier, " ".join(keynames)))
+                    for keyname in keynames:
+                        meanings[keyname] = modifier
+                return  clear, add, meanings, [], []
+        except Exception, e:
+            log.error("failed to use native get_modifier_mappings: %s", e, exc_info=True)
+        return self.modifiers_fallback()
+
+    def modifiers_fallback(self):
         xmodmap_pm = self.exec_get_keyboard_data(["xmodmap", "-pm"])
         if not xmodmap_pm:
             return ClientExtrasBase.get_keymap_modifiers(self)
         #parse it so we can feed it back to xmodmap (ala "xmodmap -pke")
         clear = []
         add = []
+        meanings = {}
         for line in xmodmap_pm.splitlines()[1:]:
             if not line:
                 continue
@@ -292,13 +319,13 @@ class ClientExtras(ClientExtrasBase):
             #ie: ['shift', 'Shift_L', '(0x32),', 'Shift_R', '(0x3e)']
             clear.append("clear %s" % parts[0])
             if len(parts)>1:
-                nohex = set([x for x in parts[1:] if not x.startswith("(")])
-                add.append("add %s = %s" % (parts[0], " ".join(nohex)))
-        log("get_keymap_modifiers parsed to clear=%s, add=%s", clear, add)
-        return  clear, add
-
-    def supports_raw_keycodes(self):
-        return True
+                nohex = [x for x in parts[1:] if not x.startswith("(")]
+                add.append("add %s = %s" % (parts[0], " ".join(set(nohex))))
+                for x in nohex:
+                    #ie: meanings['Shift_L']=shift
+                    meanings[x] = parts[0]
+        log.debug("get_keymap_modifiers parsed: clear=%s, add=%s, meanings=%s", clear, add, meanings)
+        return  clear, add, meanings, [], []
 
     def get_keymap_spec(self):
         xkbmap_print = self.exec_get_keyboard_data(["setxkbmap", "-print"])
@@ -308,7 +335,7 @@ class ClientExtras(ClientExtrasBase):
         if xkbmap_query is None and xkbmap_print is not None:
             log.error("the server will try to guess your keyboard mapping, which works reasonably well in most cases");
             log.error("however, upgrading 'setxkbmap' to a version that supports the '-query' parameter is preferred");
-        xmodmap_data = self.exec_get_keyboard_data(["xmodmap", "-pke"]);
+        xmodmap_data = self.exec_get_keyboard_data(["xmodmap", "-pke"])
         return xkbmap_print, xkbmap_query, xmodmap_data
 
     def get_keyboard_repeat(self):
@@ -320,5 +347,9 @@ class ClientExtras(ClientExtrasBase):
             log.error("failed to get keyboard repeat rate: %s", e)
         return None
 
-    def grok_modifier_map(self, display_source):
-        return grok_modifier_map(display_source)
+    def grok_modifier_map(self, display_source, xkbmap_mod_meanings):
+        try:
+            from wimpiggy.keys import grok_modifier_map
+            return grok_modifier_map(display_source, xkbmap_mod_meanings)
+        except ImportError:
+            return ClientExtrasBase.grok_modifier_map(self, display_source, xkbmap_mod_meanings)
